/*
  Project: Expanding I/O lines with MCP23008 port expander
  MCU: PIC12F683 @ 8.0 MHz internal clock, MCLR disabled
  Copyright @ Rajendra Bhatt, Jun 5, 2011
*/

// Software I2C connections
sbit Soft_I2C_Scl at GP0_bit;
sbit Soft_I2C_Sda at GP1_bit;
sbit Soft_I2C_Scl_Direction at TRISIO0_bit;
sbit Soft_I2C_Sda_Direction at TRISIO1_bit;
// End Software I2C connections

sbit IntPin at GP2_bit;

void Init_MCP23008(){
 Delay_ms(10);
 Soft_I2C_Start(); // Issue start signal
 Soft_I2C_Write(0x40); // Address + Write bit
 Soft_I2C_Write(0x00); // Select IODIR register
 Soft_I2C_Write(0x80); // GP0-GP6 O/P, GP7 I/P
 Soft_I2C_Write(0x00); // IPOL
 Soft_I2C_Write(0x80); // GP7 Interrupt-on-change
 Soft_I2C_Write(0x80); // GP7 Def value 1
 Soft_I2C_Write(0x80); // Interrupts on GP7 compares to DEFVAL
 Soft_I2C_Write(0x22); // Disable further sequential write operation
 Soft_I2C_Stop();
 
 // Enable pull-up on GP7
 Soft_I2C_Start(); // Issue start signal
 Soft_I2C_Write(0x40); // Address + Write bit
 Soft_I2C_Write(0x06); // Select IODIR register
 Soft_I2C_Write(0x80); // GP7 pull-up
 Soft_I2C_Stop();
 
}
// Look up table for seven segment LEDs
unsigned short mask(unsigned short temp) {
  switch (temp) {
    case 0 : return 0x3F;
    case 1 : return 0x06;
    case 2 : return 0x5B;
    case 3 : return 0x4F;
    case 4 : return 0x66;
    case 5 : return 0x6D;
    case 6 : return 0x7D;
    case 7 : return 0x07;
    case 8 : return 0x7F;
    case 9 : return 0x6F;
  } //case end
}

unsigned short i, k, disp, num = 0;
void main() {
 CMCON0 = 7;  // Disable Comparators
 TRISIO = 0b00001100;       // All Outputs, except GP2 and GP3
 ANSEL  = 0b00000000;       // All pins digital I/O

 Soft_I2C_Init();
 Init_MCP23008();

 do {
 while (IntPin == 0)
 Delay_us(1);
 // Read INTCAP
  Soft_I2C_Start(); // Issue start signal
  Soft_I2C_Write(0x40); // Address + Write bit
  Soft_I2C_Write(0x08);   // Address of INTCAP register
  //Soft_I2C_Stop();
  Soft_I2C_Start(); // Issue start signal
  Soft_I2C_Write(0x40+1); // Address + Read bit
  Delay_ms(300);
  k = Soft_I2C_Read(0); // Read byte and don't acknowlegde
  Soft_I2C_Stop();
  disp = mask(num);
  Soft_I2C_Start(); // Issue start signal
  Soft_I2C_Write(0x40); // Address + Write bit
  Soft_I2C_Write(0x09); // Select GPIO register
  Soft_I2C_Write(disp); // Send Num
  Soft_I2C_Stop();
  num = num + 1;
  if(num == 10) num=0;
 } while (1);
}